// Protocol specification tests
// These tests were originally in src/protocols/spec.rs and have been moved here
// to reduce the size of that file and improve test organization.

mod chat_completion;
mod chat_message;
mod embedding;
mod rerank;
mod responses;
