// Protocol definitions and validation for various LLM APIs
// This module provides a structured approach to handling different API protocols

pub mod builders;
pub mod chat;
pub mod classify;
pub mod common;
pub mod completion;
pub mod embedding;
pub mod event_types;
pub mod generate;
pub mod rerank;
pub mod responses;
pub mod sampling_params;
pub mod validated;
pub mod worker_spec;
