//! Regular (non-harmony) model processing
//!
//! This module contains all code specific to regular tokenizer-based models,
//! including pipeline stages, response processing, and streaming.

pub mod processor;
pub mod responses;
pub mod stages;
pub mod streaming;
